"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var GraphCluster_1 = require("../../graph/GraphCluster");
var GraphGenerator_1 = require("../../graph/GraphGenerator");
var GraphNode_1 = require("../../graph/GraphNode");
var GraphOptimizer_1 = require("../../graph/utils/GraphOptimizer");
var GraphVisitor_1 = require("../../graph/utils/GraphVisitor");
var DateHelper_1 = require("../../utils/DateHelper");
var DocGeneratorBase_1 = require("../shared/DocGeneratorBase");
var DotStyleGenerator_1 = require("./utils/DotStyleGenerator");
var MapNameToId_1 = require("./utils/MapNameToId");
var DotDocGenerator = /** @class */ (function (_super) {
    __extends(DotDocGenerator, _super);
    function DotDocGenerator(config, outputter) {
        var _this = _super.call(this, config, outputter) || this;
        _this.config = config;
        _this.outputter = outputter;
        _this.mapNameToId = new MapNameToId_1.MapNameToId();
        _this.styler = new DotStyleGenerator_1.DotStyleGenerator(config, outputter);
        return _this;
    }
    DotDocGenerator.prototype.generateDoc = function (packageConfig) {
        var _this = this;
        var graph = this.generateGraph(packageConfig);
        this.outputSectionSeparator("Header");
        this.outputHeader();
        this.outputter.outputLine("");
        this.outputSectionSeparator("Styling");
        this.styler.outputStyling();
        this.outputter.outputLine("");
        this.outputSectionSeparator("Nodes and Clusters");
        graph.nodes.forEach(function (node) {
            if (node instanceof GraphCluster_1.GraphCluster) {
                _this.outputCluster(node);
                return;
            }
            if (node instanceof GraphNode_1.GraphNode) {
                _this.outputGraphNode(node);
            }
        });
        this.outputter.outputLine("");
        this.outputSectionSeparator("Edges");
        this.outputEdges(graph);
        this.outputter.outputLine("");
        this.outputFooter();
    };
    DotDocGenerator.prototype.generateGraph = function (packageConfig) {
        var generator = new GraphGenerator_1.GraphGenerator(this.config, this.filter);
        var graph = generator.generateGraph(packageConfig);
        if (this.config.dot.isGraphOptimizerEnabled) {
            var optimizer = new GraphOptimizer_1.GraphOptimizer();
            optimizer.optimize(graph);
        }
        return graph;
    };
    DotDocGenerator.prototype.outputHeader = function () {
        this.outputter.outputLine("/* auto-generated by tslint-folders-diagrams docs tool at " + DateHelper_1.DateHelper.nowHumanReadable() + "*/");
        this.outputter.outputLine("digraph packages {");
        this.outputter.increaseIndent();
        this.outputGraphSettings();
    };
    DotDocGenerator.prototype.outputGraphSettings = function () {
        this.outputSectionSeparator("Graph settings");
        this.outputter.outputLine("graph [");
        this.outputter.increaseIndent();
        this.outputter.outputLines([
            "// needed to allow edge to have *cluster* as a destination:",
            "compound=true",
            // NOT using concentrate, as tends to combine edges from within a cluster, with edges from the cluster,
            // so user cannot see which edges are only from the cluster.
            "label = \"" + this.config.dot.title + "\"",
            "labelloc = t",
            "",
            "//dpi = 200",
            "ranksep=0.65",
            "nodesep=0.40",
            // top to bottom:
            "rankdir=TB",
            "",
            "style=\"filled\"",
            "",
            "len=0"
        ]);
        this.outputter.decreaseIndent();
        this.outputter.outputLine("]");
    };
    DotDocGenerator.prototype.outputFooter = function () {
        this.outputter.decreaseIndent();
        this.outputter.outputLine("}");
    };
    DotDocGenerator.prototype.outputSectionSeparator = function (sectionName) {
        this.outputter.outputLine("/* " + sectionName + " ================================= */");
    };
    DotDocGenerator.prototype.outputGraphNode = function (node, prefix) {
        this.outputScopeBegin();
        this.styler.outputGraphNodeStyle(node.nodeType);
        this.outputNode(node, prefix);
        this.outputScopeEnd();
        this.outputter.outputLine("");
    };
    DotDocGenerator.prototype.outputScopeBegin = function () {
        this.outputter.outputLine("{");
    };
    DotDocGenerator.prototype.outputScopeEnd = function () {
        this.outputter.outputLine("}");
    };
    /**
     * Output the separate graph cluster.
     */
    DotDocGenerator.prototype.outputCluster = function (cluster) {
        var isContainer = cluster.nodes.length > 0;
        if (!isContainer) {
            return;
        }
        this.outputContainerNodeStart(cluster);
        this.outputSubNodes(cluster);
        this.outputContainerNodeEnd();
        this.outputter.outputLine("");
    };
    DotDocGenerator.prototype.outputContainerNodeStart = function (cluster) {
        this.outputter.outputLine("subgraph cluster_" + cluster.id + " {");
        this.outputter.increaseIndent();
        this.styler.outputContainerStyle(cluster.clusterType);
        var formattedDescription = cluster.description.length > 0 ? " - " + cluster.description : "";
        this.outputter.outputLine("label = \"" + cluster.name + formattedDescription + "\"");
    };
    DotDocGenerator.prototype.outputContainerNodeEnd = function () {
        // styling at END so is not broken by contained node styling
        this.outputter.decreaseIndent();
        this.outputter.outputLine("}");
    };
    DotDocGenerator.prototype.outputNode = function (node, prefix) {
        var packageIdKey = this.getPrefixedPackageId(node.id, prefix);
        var fillColor = this.getColorNumber(packageIdKey);
        this.outputter.outputLine(node.id + " [label=\"" + node.name + "\n" + node.description + "\" fillcolor=" + fillColor + "]");
    };
    DotDocGenerator.prototype.getColorNumber = function (packageIdKey) {
        // TODO xxx review
        if (!this.mapNameToId.hasId(packageIdKey)) {
            this.mapNameToId.getId(packageIdKey);
        }
        var packageNumber = this.mapNameToId.getNumberOrThrow(packageIdKey);
        if (packageNumber > this.config.dot.maxColors) {
            packageNumber = packageNumber % this.config.dot.maxColors;
        }
        if (packageNumber === 0) {
            // for a graphviz color scheme, the color number must be 1 based:
            packageNumber = 1;
        }
        return packageNumber;
    };
    DotDocGenerator.prototype.getPrefixedPackageId = function (packageName, prefix) {
        return prefix + "_" + packageName;
    };
    DotDocGenerator.prototype.outputEdges = function (root) {
        var _this = this;
        var visitor = new GraphVisitor_1.GraphVisitor(root);
        visitor.visitEdges(function (edge) { return _this.outputEdge(edge); });
    };
    DotDocGenerator.prototype.outputEdge = function (edge) {
        var destination = edge.destination;
        var origin = edge.origin;
        // set a dummy label, to make edges further apart.
        // this is to avoid edges from within a cluster, seeming to have same origin
        var extraAttribute = "label=\" \" ";
        if (edge.destination instanceof GraphCluster_1.GraphCluster) {
            // the destination needs to switch to some node in the cluster:
            var nodes = edge.destination.nodes;
            if (nodes.length === 0) {
                console.warn("// warning: The cluster " + edge.destination.id + " is empty, but is the destination for an edge.");
                return;
            }
            destination = nodes[0];
            extraAttribute += " lhead=cluster_" + edge.destination.id;
        }
        if (edge.origin instanceof GraphCluster_1.GraphCluster) {
            var nodes = edge.origin.nodes;
            if (nodes.length === 0) {
                console.warn("// warning: The cluster " + edge.origin.id + " is empty, but is the origin for an edge.");
                return;
            }
            origin = nodes[0];
            extraAttribute += " ltail=cluster_" + edge.origin.id;
        }
        this.outputter.outputLine(origin.id + "-> " + destination.id + " [" + extraAttribute + "]");
    };
    DotDocGenerator.prototype.outputSubNodes = function (cluster) {
        var _this = this;
        if (cluster.nodes.length === 0) {
            this.outputter.outputLine("");
            return;
        }
        cluster.nodes.forEach(function (node) {
            if (node instanceof GraphCluster_1.GraphCluster) {
                _this.outputCluster(node);
                return;
            }
            _this.outputGraphNode(node, cluster.name);
            _this.outputter.outputLine("");
        });
    };
    return DotDocGenerator;
}(DocGeneratorBase_1.DocGeneratorBase));
exports.DotDocGenerator = DotDocGenerator;
//# sourceMappingURL=DotDocGenerator.js.map