"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var GraphCluster_1 = require("../../../graph/GraphCluster");
var GraphNode_1 = require("../../../graph/GraphNode");
var DotStyleGenerator = /** @class */ (function () {
    function DotStyleGenerator(config, outputter) {
        this.config = config;
        this.outputter = outputter;
    }
    DotStyleGenerator.prototype.outputStyling = function () {
        this.outputGraphStyle();
        this.outputDefaultNodeStyling();
    };
    DotStyleGenerator.prototype.outputGraphNodeStyle = function (nodeType) {
        switch (nodeType) {
            case GraphNode_1.NodeType.Normal:
                break;
            case GraphNode_1.NodeType.External:
                this.outputStylingForExternalNode();
                break;
            case GraphNode_1.NodeType.Any:
                this.outputStylingForAnyNode();
                break;
            default:
                throw new Error("unhandled NodeType " + nodeType);
        }
    };
    DotStyleGenerator.prototype.outputStylingForExternalNode = function () {
        this.outputter.outputLine("node [style=dashed]");
    };
    DotStyleGenerator.prototype.outputStylingForAnyNode = function () {
        this.outputStylingForExternalNode();
    };
    DotStyleGenerator.prototype.outputContainerStyle = function (clusterType) {
        this.outputPlaceLabelsAtTop();
        switch (clusterType) {
            case GraphCluster_1.ClusterType.AreaWithSubFolders:
                this.outputter.outputLine("node [shape=\"" + this.config.dot.subFolderShape + "\"]");
                break;
            case GraphCluster_1.ClusterType.FromOptimization:
                this.outputOptimizedStyle();
                break;
            case GraphCluster_1.ClusterType.DiagramCluster:
                this.outputOptimizedStyle();
                break;
            case GraphCluster_1.ClusterType.Root:
            case GraphCluster_1.ClusterType.TopLevel:
                this.outputter.outputLine("node [shape=\"" + this.config.dot.packageShape + "\"]");
                break;
            default:
                throw new Error("unhandled ClusterType " + clusterType);
        }
    };
    DotStyleGenerator.prototype.outputOptimizedStyle = function () {
        this.outputter.outputLines(["color = gray", "style=dashed"]);
    };
    DotStyleGenerator.prototype.outputPlaceLabelsAtTop = function () {
        this.outputter.outputLine("labelloc = t");
        this.outputter.outputLine("");
    };
    DotStyleGenerator.prototype.outputGraphStyle = function () {
        this.outputter.outputLine("graph [");
        this.outputter.increaseIndent();
        this.outputter.outputLines(["bgcolor=\"#FFFFFF\"", "fillcolor=\"#FFFFFF\""]);
        this.outputter.decreaseIndent();
        this.outputter.outputLine("]");
    };
    DotStyleGenerator.prototype.outputDefaultNodeStyling = function () {
        // ref: working_copy.dot
        this.outputter.outputLine("node [");
        this.outputter.increaseIndent();
        // ref: colorscheme = https://graphviz.gitlab.io/_pages/doc/info/colors.html
        this.outputter.outputLines([
            "labeljust=\"l\"",
            "colorscheme=\"" + this.config.dot.colorScheme + "\"",
            "style=filled",
            "fillcolor=3",
            "shape=record"
        ]);
        this.outputter.decreaseIndent();
        this.outputter.outputLines([
            "]",
            "",
            // setting both head, tail as normally only one will apply
            "edge [arrowhead=\"vee\", arrowtail=\"vee\", style=dashed, color=\"black\"]"
        ]);
    };
    return DotStyleGenerator;
}());
exports.DotStyleGenerator = DotStyleGenerator;
//# sourceMappingURL=DotStyleGenerator.js.map