"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var ConfigDefaults_1 = require("../ConfigDefaults");
var EnumUtils_1 = require("./EnumUtils");
var NUM_MANDATORY_ARGS = 4;
var HelpNeededError = /** @class */ (function (_super) {
    __extends(HelpNeededError, _super);
    function HelpNeededError() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return HelpNeededError;
}(Error));
var ArgsParser;
(function (ArgsParser) {
    function getConfigFromArgs() {
        if (process.argv.length < NUM_MANDATORY_ARGS) {
            return null;
        }
        try {
            var config = ConfigDefaults_1.ConfigDefaults.getDefault();
            updateConfigFromArgs(config);
            updateConfigFromOptionalArgs(config);
            return config;
        }
        catch (error) {
            if (error instanceof HelpNeededError) {
                return null;
            }
            console.error(error);
            return null;
        }
    }
    ArgsParser.getConfigFromArgs = getConfigFromArgs;
    function updateConfigFromArgs(config) {
        var formatString = process.argv[3];
        var format = EnumUtils_1.EnumUtils.parseDocFormat(formatString);
        Object.assign(config, {
            pathToTslintJson: process.argv[2],
            format: format
        });
    }
    function updateConfigFromOptionalArgs(config) {
        var _loop_1 = function (i) {
            var optionArg = process.argv[i];
            var optionParts = optionArg.split("=");
            var option = optionParts[0].trim();
            var value = optionParts.length > 1 ? optionParts[1].trim() : null;
            var assertHasValue = function (message) {
                if (!value) {
                    throw new Error(message);
                }
                return value;
            };
            var assertHasNumericValue = function (message) {
                var textValue = assertHasValue(message);
                var numberValue = Number.parseInt(textValue, 10);
                if (!isFinite(numberValue)) {
                    throw new Error(message);
                }
                return numberValue;
            };
            switch (option) {
                case "-clusterFromTslintJson":
                    config.dot.clusterFromTslintJson = true;
                    break;
                case "-colorScheme":
                    value = assertHasValue("color scheme must have a value, like: colorScheme=piyg11");
                    config.dot.colorScheme = value;
                    break;
                case "-disableGraphOptimizer":
                    config.dot.isGraphOptimizerEnabled = false;
                    break;
                case "-h":
                case "-help":
                    throw new HelpNeededError();
                case "-importBlacklist":
                    value = assertHasValue("black list must have a value, like: importBlacklist=todo-area,contact-area");
                    config.importBlacklist = value;
                    break;
                case "-importWhitelist":
                    value = assertHasValue("white list must have a value, like: importWhitelist=todo-area,contact-area");
                    config.importWhitelist = value;
                    break;
                case "-maxColors":
                    var maxColors = assertHasNumericValue("max colors must have a value, like: maxColors=8");
                    config.dot.maxColors = maxColors;
                    break;
                case "-outpath":
                    value = assertHasValue("outpath must have a value, like: outpath=/tmp/my-file.dot");
                    config.outpath = value;
                    break;
                case "-packageShape": {
                    value = assertHasValue("package shape must have a 'graphviz shape' value, like: packageShape=component");
                    config.dot.packageShape = value;
                    break;
                }
                case "-showImportAnyAsNodeNotEdges":
                    config.dot.showImportAnyAsNodeNotEdges = true;
                    break;
                case "-skipSubFolders":
                    config.skipSubFolders = true;
                    break;
                case "-subFolderShape": {
                    value = assertHasValue("sub-folder shape must have a 'graphviz shape' value, like: subFolderShape=octagon");
                    config.dot.subFolderShape = value;
                    break;
                }
                case "-subTitle": {
                    value = assertHasValue("sub title must have a value, like: subTitle=MyTitle");
                    config.dot.subTitle = value;
                    break;
                }
                case "-title": {
                    value = assertHasValue("title must have a value, like: title=MyTitle");
                    config.dot.title = value;
                    break;
                }
                case "":
                    break;
                default:
                    throw new Error("unrecognised option " + optionArg);
            }
        };
        for (var i = NUM_MANDATORY_ARGS; i < process.argv.length; i++) {
            _loop_1(i);
        }
    }
})(ArgsParser = exports.ArgsParser || (exports.ArgsParser = {}));
//# sourceMappingURL=ArgsParser.js.map