"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var PackageFilter = /** @class */ (function () {
    function PackageFilter(config) {
        this.invalidPaths = this.parseBlacklist(config.importBlacklist);
        this.whiteListPaths = this.parseWhitelist(config.importWhitelist);
        this.skipSubFolders = config.skipSubFolders;
    }
    PackageFilter.prototype.parseBlacklist = function (list) {
        return list.split(",");
    };
    PackageFilter.prototype.parseWhitelist = function (list) {
        return list.split(",").filter(function (entry) { return entry.length > 0; });
    };
    PackageFilter.prototype.passesBlacklist = function (importPath) {
        return this.invalidPaths.indexOf(importPath) < 0;
    };
    PackageFilter.prototype.passesWhitelist = function (importPath) {
        return this.whiteListPaths.length === 0 || this.whiteListPaths.indexOf(importPath) >= 0;
    };
    PackageFilter.prototype.canOutputSubfoldersOf = function (folder) {
        return this.passesWhitelist(folder.importPath);
    };
    Object.defineProperty(PackageFilter.prototype, "hasWhitelist", {
        get: function () {
            return this.whiteListPaths.length > 0;
        },
        enumerable: true,
        configurable: true
    });
    PackageFilter.prototype.isImportPathOkForFolder = function (folder) {
        return this.passesBlacklist(folder.importPath);
    };
    PackageFilter.prototype.isImportPathOkForFolderWithWhitelist = function (folder) {
        return this.isImportPathOkForFolder(folder) && this.passesWhitelist(folder.importPath);
    };
    PackageFilter.prototype.isImportPathOkForSubFolder = function (folder) {
        return !this.skipSubFolders && this.passesBlacklist(folder.importPath);
    };
    PackageFilter.prototype.isImportPathOkForEdges = function (importPath) {
        return this.passesBlacklist(importPath);
    };
    return PackageFilter;
}());
exports.PackageFilter = PackageFilter;
//# sourceMappingURL=PackageFilter.js.map